 //////////////////////////////////////////////////////////////////////////////
//
// PocketFrog - The Game Library for Pocket PC Devices
// Copyright 2002  Thierry Tremblay
//
// Permission to use, copy, modify, distribute and sell this software
// and its documentation for any purpose is hereby granted without fee,
// provided that the above copyright notice appear in all copies and
// that both that copyright notice and this permission notice appear
// in supporting documentation.  Thierry Tremblay makes no representations
// about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
//////////////////////////////////////////////////////////////////////////////

#include "blit.h"
#include "resource.h"

using namespace Frog;

#include <windef.h>



//////////////////////////////////////////////////////////////////////////////
//
// _Module : This is needed to use ATL Windows
//
//////////////////////////////////////////////////////////////////////////////

CComModule _Module;


//////////////////////////////////////////////////////////////////////////////
//
// BlitSample
//
//////////////////////////////////////////////////////////////////////////////




BlitSample::BlitSample()
{

    m_config.szWindowName     = _T("Blit Sample");
    m_config.orientation      = ORIENTATION_NORTH;
    m_config.splashScreenTime = 100;

}


void BlitSample::ResetPictureMelt(void)
{




	for(int i=0; i<240/COLSIZE; i++)
	{
		slidecolumns[i] = 0;
		switch(melttype)
		{
		case 1:
			slidespeeds[i] = MINCOLSPEED + fabs((MAXCOLSPEED - MINCOLSPEED) * sin(i* 3.14f/(240/COLSIZE) ));
			break;
		case 2:
			slidespeeds[i] = MINCOLSPEED + fabs((MAXCOLSPEED - MINCOLSPEED) * cos(i* 3.14f/(240/COLSIZE) ));
			break;
		case 3:
			slidespeeds[i] = 0.0f;
			slidespeeds[0] = MAXCOLSPEED;
			break;
		default:
			slidespeeds[i] = MINCOLSPEED + (MAXCOLSPEED - MINCOLSPEED) * rand()/RAND_MAX;
		}
	}

}

int BlitSample::DoPictureMelt(DisplayDevice* display, Surface * s1, Surface * s2)
{
static int col=0;
int offset;
Rect srcrect;


	int stillgoing = 0;


	for(int i=0; i<ITERSPERFRAME; i++)
	{
		



		if( (++col) >= 240/COLSIZE ) col = 0;

		if(melttype == 3)
		{
#define AVGSPEED	0.55f
			if(col>0)
			  slidespeeds[col] = AVGSPEED*slidespeeds[col] + (1.0f-AVGSPEED)*slidespeeds[col-1];
		}

		if(slidecolumns[col] < 500)
		{

			offset = slidecolumns[col]+= slidespeeds[col];

			if(offset>=320)
			{
				if(offset<500)
				{
					srcrect.Set(col*COLSIZE, 0, (col+1)*COLSIZE, 319);
					display->Blit( col*COLSIZE, 0, s2, &srcrect );
					slidecolumns[col] = 500;
				}


			}

			if(offset<319 && offset > 0)
			{
				stillgoing = 1;

				srcrect.Set(col*COLSIZE, 0, (col+1)*COLSIZE, 319-offset);
				display->Blit( col*COLSIZE, offset, s1, &srcrect );
				
				srcrect.Set(col*COLSIZE, 0, (col+1)*COLSIZE, offset);
				display->Blit( col*COLSIZE, 0, s2, &srcrect );
			}
			
		}


	}

	if(stillgoing)
		return 0;
	else
		return -1;
}



bool BlitSample::GameInit()
{



		// Initialize random seed
    srand( PocketPC::GetTickCount() );
    
    DisplayDevice* display = GetDisplay();
    

	ResetPictureMelt();
	melttype = 0;
    
	sp1 = m_background = LoadImage( display, IDR_IMAGE1 );
 	sp2 = m_background2 = LoadImage( display, IDR_IMAGE7 );
   
     
    
    m_numbers = LoadImage( display, IDB_NUMBERS );
    m_FPSCounter = 0;

//    m_dragImage = -1;   // Not dragging any image


    
	display->Blit( 0, 0, sp1 );
	display->Update();


    return true;

}



void BlitSample::GameEnd()
{

    
    delete m_background;
    delete m_numbers;

}



void BlitSample::GameLoop()
{

    DisplayDevice* display = GetDisplay();

	
	
    if(DoPictureMelt(display, sp1, sp2) < 0)
	{	//swap images

		if( (++melttype) >= 4 ) melttype=0;
		
		ResetPictureMelt();

		Surface * temp = sp2;
		sp2 = sp1;
		sp1 = temp;

	}


	






    // Update FPS
    m_FPSTicks[ m_FPSCounter & 15 ] = PocketPC::GetTickCount();
    if (m_FPSCounter > 15)
    {
        uint32_t totalTime = m_FPSTicks[ m_FPSCounter & 15 ] - m_FPSTicks[ (m_FPSCounter+1) & 15 ];
        if (totalTime == 0) totalTime = 1;
        uint32_t fps  = 16000 / totalTime;
        uint32_t n100 = fps / 100; fps -= n100 * 100;
        uint32_t n10  = fps / 10;  fps -= n10 * 10;
        uint32_t n1   = fps;

        if (n100 > 9) { n100 = 9; n10 = 9; n1 = 9; }
        
        // Display fps
        display->Blit(  0, 0, m_numbers, &Rect( n100 * 8, 0, (n100+1)*8, 11 ) );
        display->Blit(  8, 0, m_numbers, &Rect( n10 * 8, 0, (n10+1)*8, 11 ) );
        display->Blit( 16, 0, m_numbers, &Rect( n1 * 8, 0, (n1+1)*8, 11 ) );
    }
    ++m_FPSCounter;

    // Update screen
    display->Update();

}



void BlitSample::StylusDown( Point stylus )
{


}




void BlitSample::StylusUp( Point stylus )
{


}



void BlitSample::StylusMove( Point stylus )
{


}




//////////////////////////////////////////////////////////////////////////////
//
// WinMain - Entry point
//
//////////////////////////////////////////////////////////////////////////////

int WINAPI WinMain( HINSTANCE hInstance, HINSTANCE, LPTSTR, int )
{
    _Module.Init( 0, hInstance );
    
    BlitSample game;
    
    game.Run();
    
    return 0;
}
